\version "2.14.0"

\header {
  texidoc = "
Typing music for many consecutive tremolos with the same base duration
becomes tedious. This is a function to make this easier. By default
LilyPond does not unfold tremolo repeats for midi. A function for
unfolding only tremolo repeats is also included. 

"
  doctitle = "Consecutive tremolos"
}
% tremolo functions
#(define (tremolo-repeat-count dur music)
   (let* ((elements (ly:music-property music 'elements))
          (music-dur (ly:music-property (car elements) 'duration))
          (length (ly:duration-log music-dur))
          (dots (ly:duration-dot-count music-dur))
          (beats (* (- 2 (/ 1 (expt 2 dots))) (/ 4 (expt 2 length)))))
     (* beats (/ dur 4))))

#(define (make-tremolo dur music)
   (make-music 'TremoloRepeatedMusic
               'elements '()
               'repeat-count (tremolo-repeat-count dur music)
               'tremolo-type dur
               'element music))

#(define (tremoloize dur music)
   (if (eq? (ly:music-property music 'name) 'EventChord)
       (make-tremolo dur music)
       music))

%\tremolos <base duration> <music>
%Makes each note in music a tremolo repeat.
%dur is 8, 16, 32, etc.
tremolos =
#(define-music-function (parser location dur mus) (integer? ly:music?)
   (music-map (lambda (x) (tremoloize dur x)) mus))

#(define (unfold-tremolos mus)
   (if (eq? (ly:music-property mus 'name) 'TremoloRepeatedMusic)
       (unfold-repeats mus)
       mus))

%Unfolds only tremolo repeats.
unfoldTremolos =
#(define-music-function (parser location mus) (ly:music?)
   (music-map unfold-tremolos mus))
% end tremolo functions

music = \relative c' {
  \tremolos #16 { c4 d2 e4 | f1 | g8 f e d c2 | }
}

\score {
  \new Staff { \music }
}

\score {
  \new Staff \relative c' {
    \unfoldTremolos \music
  }
  \midi { }
}
