\version "2.14.0"

\header {
  texidoc = "
The music theorist Heinrich Schenker created a method of analysing
tonal music using, what are now known as Schenker graphs or diagrams.
These use @qq{normal} musical notation in non-standard ways.

Major problems creating a Schenker graph  with lilypond are:


* Precise alignment of notes between staves needed, * Note heads,
stems, beams etc are used to notate musical structure not rhythm, * A
note head may have more than one stem or none, * Slurs may be nested
arbitrarily and can connect to both left and right of the same note
head, * Harmonic connection lines may be drawn between staves.


Solutions:


* By default all notes are eight notes and the (invisible) time
signature is 1/8. A @qq{missing} note head is implemented as a
transparent note rather than a skip. * Most grobs are, by default,
invisible. Each staff is structured as a series of @qq{layers},
implemented as voices, containing all the notes, NoteCollision
#'merge-differently-headed is set to true. Note heads are only visible
in the @qq{notes} voice, note heads are determined by setting NoteHead
#'duration-log. * Harmonic connections are implemented by a note
followers in separate @qq{layers}. 

"
  doctitle = "Creating a Schenker Graph"
}
\score {
  \context PianoStaff = "Schenker" <<
    \new Staff = "Right" {
      \clef "treble"
      \key g \major
      \time 1/8
      <<
        %% Notes
        %% It is easier to add the markup to this layer after the remainder
        %% of the graph is finished
        {
          \override NoteHead #'transparent = ##f
          g'8 fis'_\markup{\hspace #-.7 \raise #-2 {N}}
          g' a'^\markup{\hspace #-.7 \raise #0.0 {P}}
          \once \override  NoteHead #'duration-log = 1
          b' a' b'^\markup{P}
          c'' b'_\markup{P}
          \once \override NoteHead #'transparent = ##t
          e'
          \once \override  NoteHead #'duration-log = 1
          a' c'' b'_\markup{P} a'
          \once \override  NoteHead #'duration-log = 1
          g'
        }
        %% Ursatz Urlinie
        \\
        {
          \stemUp
          g'8  fis'  g'  a'
          \once \override Stem #'transparent = ##f
          \once \override Beam #'transparent = ##f
          \once \override Beam #'positions = #'(10 . 10)
          b'[ ^\markup { \override #'(baseline-skip . 0.5) \column { \small {^ 3} } }
              a'  b'  c''  b'  e'
              \once \override Stem #'transparent = ##f
              a'8^\markup { \override #'(baseline-skip . 0.5) \column { \small {^ 2} } }
              c''  b'  a'
              \once \override Stem #'transparent = ##f
              g']^\markup { \override #'(baseline-skip . 0.5) \column { \small {^ 1} } }
        }
        %% Extra stems
        \\
        {
          \stemUp
          g'8  fis'  g'  a'  b'  a'  b'
          \once \override Stem #'transparent = ##f
          \once \override Stem #'stem-end-position = #12.5
          c''
          \once \override Stem #'transparent = ##f
          \once \override Beam #'positions = #'(8.0 . 8.0)
          b'[  e']  a'  c''  b'  a' g'
        }
        %% Unfolding symbols
        \\
        {
          \once \override Beam #'positions = #'(4.0 . -3.5)
          \once \override Beam #'transparent = ##f
          \once \override Stem #'transparent = ##f
          \stemUp g'8[
            fis'  g'  a'
            \once \override Stem #'transparent = ##f
            \stemDown b'8]
          \once \override Beam #'positions = #'(3.5 . -3.5)
          \once \override Beam #'transparent = ##f
          \once \override Stem #'transparent = ##f
          \stemUp a'8[
            b'
            \once \override Stem #'transparent = ##f
            \stemDown c''] b'  e'  a'  c''  b'  a' g'
        }
        %% Upward slurs 1
        %% Most slurs are OK "as is", however some will need tweaking.
        %% Note: 1] Adding more "layers" of slurs can affect the placement of these
        %%          so the layer may need to be re-visited.
        %%       2] Don't use phrasing slurs as these can affect the alignment of notes
        {
          \slurUp
          g'8
          \once \override Slur #'control-points  = #'((0.8 . -0.7) (3 .  0) (5.5 . 0) (7 . -0.5))
          fis'(  g')  a'(  b')  a'  b'(  c'')  b'  e'  a'  c''(  b'  a') g'
        }
        %% Upward slurs 2
        \\
        {
          \slurUp
          g'8  fis'  g'  a'  b'  a'  b'
          \once \override Slur #'control-points  = #'((2 . 1.5) (2.5 . 2) (4.8 . 2.5) (7 . 1))
          c''(  b')  e'  a'  c''  b'  a' g'
        }
        %% Downward Slurs 1
        \\
        {
          \slurDown
          \once \override Slur #'extra-offset = #'(0.0 . 2.0)
          g'8(  fis'  g') a' b' a'  b' c'' b' s a' c'' b'(  a') g'
        }
        %% Downward Slurs 2
        \\
        {
          \slurDown
          \slurDashed
          g'8  fis'  g'  a'
          \once \override Slur #'height-limit = #12
          b'( a'  b' c'' b') \skip 8
          \once \override Slur #'height-limit = #8
          a'( c'' b' a') g'
        }
        %% Harmonic connection 1
        %% These are the only voices that use notes other than eight notes
        %% Fortunately using them here does not affect note placement.
        %% Note the use of the non-existent note "g8" to affect placement of
        %% part follower.
        \\
        {
          g'8  fis'  g'  a'  b'  a'4
          \change Staff = "Left"
          g8 g, b, c a, d d, g,
        }
        %% Harmonic connection 2
        \\
        {
          g'8  fis'  g'  a'  b'  a'  b'  c''  b'  e'  a'4
          \change Staff = "Left"
          g8 d, g,
        }
      >>
      \bar "|."
    }
    \new Staff = "Left" {
      \clef "bass"
      \time 1/8
      \key g \major
      <<
        %% Notes
        {
          \override NoteHead #'transparent = ##f
          \once \override  NoteHead #'duration-log = 1
          g8 a^\markup{P} b fis^\markup{P} e c d^\markup{N} fis, g, b, c a,
          \once \override  NoteHead #'duration-log = 1
          d d,
          \once \override  NoteHead #'duration-log = 1
          g,
        }
        %% Ursatz Bassbrechung
        \\
        {
          \stemDown
          \once \override Stem #'transparent = ##f
          \once \override Beam #'transparent = ##f
          \once \override Beam #'positions = #'(-12 . -12)
          g8[_\markup {I}
             a b fis e c d fis, g, b, c a,
             \once \override Stem #'transparent = ##f
             d_\markup {V} d,
             \once \override Stem #'transparent = ##f
             g,]_\markup {I}
        }
        %% Extra stems
        \\
        {
          \stemDown
          \shiftOff
          g8 a b fis e c d fis,
          \once \override Stem #'transparent = ##f
          \once \override Beam #'positions = #'(-10.0 . -10.0)
          g,[ b,]
          \once \override Stem #'transparent = ##f
          \once \override Stem #'stem-end-position = #-14.5
          c
          a, d d, g,
        }
        %% Unfolding symbols
        \\
        {
          g8 a b fis e
          \once \override Beam #'positions = #'(-4.5 . 1)
          \once \override Beam #'transparent = ##f
          \once \override Stem #'transparent = ##f
          \stemDown c[
            d
            \once \override Stem #'transparent = ##f
            \stemUp fis,]
          \once \override Beam #'positions = #'( 1 . -4.5)
          \once \override Beam #'transparent = ##f
          \once \override Stem #'transparent = ##f
          \stemUp g,[
            \once \override Stem #'transparent = ##f
            \stemDown b,] c a, d d, g,
        }
        %% Upward slurs 1
        \\
        {
          \slurUp
          \once \override Slur #'extra-offset = #'(-1.0 . -2.0)
          g8( a b)
          \once \override Slur #'extra-offset = #'(-1.0 . 0)
          fis( e)
          \once \override Slur #'extra-offset = #'(-1.0 . 0)
          c( d) fis, g, b,
          \once \override Slur #'extra-offset = #'(-1.0 . -1)
          c( a, d) d, g,
        }
        %% Upward slurs 2
        \\
        {
          \slurUp
          g8 a b fis e c d fis, g, b, c a,
          \once \override Slur #'control-points  = #'((2 . 0.8) (2.5 . 1.4) (6.0 . 2.2) (7.5 . -2.5))
          d( d,) g,
        }
        %% Downward slurs 1
        \\
        {
          \slurDown
          g8 a( b) fis e c d
          \once \override Slur #'extra-offset = #'(0.0 . .2)
          fis,( g,) b, c( a,) d d, g,
        }
        %% Downward slurs 2
        \\
        {
          \slurDown
          \once \override Slur #'extra-offset = #'(0.0 . 2)
          g8( a b fis e) c d fis, g, b, c a, d d, g,
        }
        %% Downward slurs 3
        \\
        {
          \slurDown
          \slurDashed
          \once \override Slur #'height-limit = #10
          g8( a b fis e c d fis, g,) b, c a, d d, g,
        }
      >>
      \bar "|."
    }
  >>
  \layout {
    indent = #0.0
    \context {
      \Score
      \remove "Bar_number_engraver"
      defaultBarType = "empty"
    }
    \context {
      \Staff
      \override NoteCollision #'merge-differently-headed = ##t
      \remove "Time_signature_engraver"
      \override Stem #'transparent = ##t
      \override Beam #'transparent = ##t
      \override NoteHead #'transparent = ##t

    }
    \context {
      \Voice
      \override NoteColumn #'ignore-collision = ##t
    }
    \context {
      \PianoStaff
      followVoice = ##t
    }
  }
}
